from typing import Union, List, Tuple

WAIFU = [
    "Littlepip",
    "Toaster Repair Pony",
    "Littlepip",
    "The Lightbringer",
    "The Stable Dweller"
]
BEST_PONY = [
    "Applejack",
    "Daring Do",
    "Littlepip",
]


""":var UNWORTHY: A list of either strings, or a fun-fact tuple."""
UNWORTHY: List[Union[str, Tuple[str, str, str]]] = [
    "Blackjack",
    "Blackjerk",
    "oc:blackjack",
    "Security Mare (Blackjack)",
    "Maiden of the Stars (Blackjack)",
    "Queen Whiskey (Blackjack)",
    "Queen Blackjack",
    "Black-And-Red OC",
    "Blackjack (Cyber-Unicorn with Cyber Wings)",
    "Blackjack (Cyber Alicorn)",
    "Blackjack (Unicorn Blank/Clone again after her original body was torn in two)",
    "Blackjack (Biological Great-great-great-great-great-great-great-great-great Granddaughter of Twilight Sparkle)",
    "Blackjack (Biological Great-great-great-great-great-great-great-great-great Granddaughter of Big Macintosh)",
    "Blackjack Clone (Made out of Discords blood)",
    ("Blackjack", "Did you know:", "Blackjack died up to 4 times:\n\nIntroduction (Alive): Chp. 1\nDead: Chp. 33\nAlive again: Chp. 34\nDead: Chp. 48\nAlive: Chp. 49\nPossibly Dead: Chp. 54\nAlive Chp.54 \nDead end of Chp.77 (Original body)\nAlive in Epilogue (Clone body, implied to be immune to age)"),
    ("Blackjack", "Did you know:", "Blackjack has had two boats dropped on her in her time in Hoofington - three if you count an airship."),
    ("Blackjack", "Did you know:", "In an interview with the fan website Equestria Daily, Somber was originally going to have Blackjack be killed off by the end of Chapter 33 while Rampage and Scotch Tape continue where she left off."),
    ("Blackjack", "Did you know:", "Another running gag is Blackjack's ignorance that Littlepip is the famed Stable Dweller."),
]
